#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import km3pipe as kp
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def get_time_array(fname, savestr = ''):
    event_pump = kp.io.hdf5.HDF5Pump(filename=fname)

    time_mean_trigg_all_events = None
    mean_triggered_time_list = []
    for i, event_blob in enumerate(event_pump):
        if i % 200 == 0:
            print('Event No. ' + str(i))

        time = event_blob['Hits'].time
        triggered = event_blob['Hits'].triggered

        mean_triggered_time = np.mean(time[triggered==1])
        mean_triggered_time_list.append(mean_triggered_time)
        time_minus_mean = np.subtract(time, mean_triggered_time)

        ax = np.newaxis
        #time_trigg = np.concatenate([time[:, ax], triggered[:, ax]], axis=1)
        time_minus_mean_trigg = np.concatenate([time_minus_mean[:, ax], triggered[:, ax]], axis=1)

        if i==0:
            time_mean_trigg_all_events = time_minus_mean_trigg
        else:
            time_mean_trigg_all_events = np.concatenate([time_mean_trigg_all_events, time_minus_mean_trigg], axis=0)

        # plotting
        plotting = False
        if plotting:
            time_only_trigg = time[triggered == 1]
            plt.hist(time_only_trigg, bins=50)
            plt.savefig('hist_time_trigg_only.png')

            plt.cla()

            plt.hist(time, bins=50)
            plt.savefig('hist_time.png')

            break

    np.save(savestr + '_time_mean_trigg_all_events.npy', time_mean_trigg_all_events)

    mean_triggered_time_arr = np.array(mean_triggered_time_list, dtype=np.float64)
    np.save(savestr + '_mean_triggered_time_arr_each_event.npy', mean_triggered_time_arr)


def get_time_array_mc_hits(fname, savestr='', mean=('', None)):
    event_pump = kp.io.hdf5.HDF5Pump(filename=fname)

    mc_hits_time_mean_all_events = None
    for i, event_blob in enumerate(event_pump):
        if i % 200 == 0:
            print('Event No. ' + str(i))

        time = event_blob['McHits'].time

        if mean[0] == 'trigger':
            mean_triggered_time_arr = mean[1]
            mean_triggered_time_event = mean_triggered_time_arr[i]
            time_minus_mean = np.subtract(time, mean_triggered_time_event)

        else:
            mean_time = np.mean(time)
            time_minus_mean = np.subtract(time, mean_time)

        if i==0:
            mc_hits_time_mean_all_events = time_minus_mean
        else:
            mc_hits_time_mean_all_events = np.concatenate([mc_hits_time_mean_all_events, time_minus_mean], axis=0)

        # plotting
        plotting = False
        if plotting:
            plt.hist(time, bins=50)
            plt.savefig('hist_mc_hits_time.png')
            break

    if mean[0] == 'trigger':
        np.save(savestr + '_mc_hits_time_meantriggered_all_events.npy', mc_hits_time_mean_all_events) # mean centering with triggered hits
    else:
        np.save(savestr + '_mc_hits_time_mean_all_events.npy', mc_hits_time_mean_all_events) # mean centering with mc hits


def plot(savestr=''):

    #triggered = time_trigg_all_events[:, 1]
    #time_only_trigg = time_trigg_all_events[triggered == 1]

    time_mean_trigg_all_events = np.load(savestr + '_time_mean_trigg_all_events.npy')

    # plotting
    plt.hist(time_mean_trigg_all_events[:, 0], bins=100)
    plt.grid(True, zorder=0, linestyle='dotted')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_time_minus_mean_all_events.png')
    plt.cla()

    plt.hist(time_mean_trigg_all_events[:, 0], bins=100, range=(-1000, 1000))
    y_lim = plt.ylim() # current one
    plt.ylim((0.6*y_lim[1], y_lim[1]))
    plt.grid(True, zorder=0, linestyle='dotted')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_time_minus_mean_all_events_zoomy_-1000_1000.png')
    plt.cla()

    plt.hist(time_mean_trigg_all_events[:, 0], bins=100, range=(-500, 500))
    plt.grid(True, zorder=0, linestyle='dotted')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_time_minus_mean_all_events_zoomy_-500_500.png')
    plt.cla()

    triggered = time_mean_trigg_all_events[:, 1]
    time_minus_mean_only_trigg_all_events = time_mean_trigg_all_events[triggered==1]

    plt.hist(time_minus_mean_only_trigg_all_events[:, 0], bins=100)
    plt.grid(True, zorder=0, linestyle='dotted')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_time_minus_mean_all_events_trigg_only_.png')


def plot_mc_hits(savestr='', mean=''):
    mc_hits_time_mean_all_events = np.load(savestr + '_mc_hits_time_mean' + mean + '_all_events.npy')

    # plotting

    plt.hist(mc_hits_time_mean_all_events, bins=100)
    plt.grid(True, zorder=0, linestyle='dotted')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_mc_hits_time_minus_mean' + mean + '_all_events.png')

    plt.cla()

    # plt.hist(mc_hits_time_mean_all_events, bins=100, range=(-2500, 2500))
    # plt.savefig(savestr + '_hist_mc_hits_time_minus_mean_all_events_zoom_-2500_2500.png')
    # plt.cla()

    plt.hist(mc_hits_time_mean_all_events, bins=100, range=(-1000, 1500))
    plt.grid(True, zorder=0, linestyle='dotted')

    plt.xlabel('MC-Hit time minus mean time of all triggered hits [ns]')
    plt.ylabel('Number of hits [#]')
    title = plt.title('MC-Hit time pattern for ', savestr, ' events')
    title.set_position([.5, 1.04])
    plt.axvline(x=-250, color='black', linestyle='--', label='Timecut 1')
    plt.axvline(x=500, color='black', linestyle='--')
    plt.axvline(x=-150, color='firebrick', linestyle='--', label='Timecut 2')
    plt.axvline(x=200, color='firebrick', linestyle='--')
    plt.legend(prop={'size': 12})
    plt.tight_layout()

    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_mc_hits_time_minus_mean' + mean + '_all_events_zoom_-1000_1500.png')
    plt.savefig('./plots/' + savestr + '/' + savestr + '_hist_mc_hits_time_minus_mean' + mean + '_all_events_zoom_-1000_1500.pdf')
    plt.cla()


if __name__ == '__main__':
    ptype = 'mupage'

    if ptype == 'mupage':
        path = '/home/saturn/capn/mppi033h/Data/raw_data/mupage/'
    elif ptype == 'random_noise':
        path = '/home/saturn/capn/mppi033h/Data/raw_data/random_noise/'
    else:
        path = '/home/woody/capn/mppi033h/Data/ORCA_JTE_NEMOWATER/raw_data/h5/calibrated/with_run_id/' + ptype + '/3-100GeV/'

    filenames = {'muon-CC': 'JTE.KM3Sim.gseagen.muon-CC.3-100GeV-9.1E7-1bin-3.0gspec.ORCA115_9m_2016.99.h5',
                 'elec-CC': 'JTE.KM3Sim.gseagen.elec-CC.3-100GeV-1.1E6-1bin-3.0gspec.ORCA115_9m_2016.99.h5',
                 'tau-CC': 'tau-CC_sample.h5',
                 'mupage': 'JTE.ph.ph.mupage.ph.ph.ph.ORCA115_9m_2016.99.h5',
                 'random_noise': ''}
    filename_input = path + filenames[ptype]

    # centered with trigg hits mean
    get_time_array(filename_input, savestr=ptype)
    plot(savestr=ptype)

    # centered with mc_hits mean
    get_time_array_mc_hits(filename_input, savestr=ptype)
    plot_mc_hits(savestr=ptype)

    # centered mc_hits with trigg hits mean for each event
    mean_triggered_time = np.load(ptype + '_mean_triggered_time_arr_each_event.npy')
    print(mean_triggered_time)
    get_time_array_mc_hits(filename_input, savestr=ptype, mean=('trigger', mean_triggered_time))
    plot_mc_hits(savestr=ptype, mean='triggered')





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Placeholder"""

import numpy as np
import h5py
import timeit
import cProfile

def generate_batches_from_hdf5_file():

    #filepath = './JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yz_NO_COMPRESSION_NOT_CHUNKED.h5' #2D, no compression
    #filepath = './JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yz_LZF_CHUNKED.h5' #2D, LZF

    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_NO_COMPRESSION_NOT_CHUNKED.h5' #3D, no compression
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_NO_COMPRESSION_CHUNKED.h5' #3D, no compression, chunked
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_LARGE_NO_COMPRESSION_CHUNKED.h5' #3D, no compression, chunked, LARGE
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_LZF_CHUNKED.h5' #3D, LZF
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_LZF_LARGE.h5' #3D, LZF, LARGE, Shuffle True
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_LZF_SHUFFLETRUE.h5' #3D, LZF, Shuffle=True
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_GZIP_COMPROPT_1.h5' #3D, GZIP
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_456_yzt_GZIP_COMPROPT_1_CHUNKSIZE_16.h5' #3D, GZIP, chunksize=16

    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_9_yzt_no_compression.h5' #3D, no compression
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_9_yzt_gzip_1.h5' # 3D, gzip, compression_opts=1


    # 4d
    filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_9_xyzt_no_compression_chunked.h5' # 4D, (11x13x18x50)), no compression. chunksize=32 --> 1011 ms
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_9_xyzt_lzf.h5' # 4D, (11x13x18x50), lzf --> 2194 ms
    #filepath = 'JTE_KM3Sim_gseagen_muon-CC_3-100GeV-9_1E7-1bin-3_0gspec_ORCA115_9m_2016_9_xyzt_gzip_1.h5' # 4D, (11x13x18x50), gzip, compression_opts=1 --> 1655 ms


    #print 'Testing generator on file ' + filepath
    #batchsize = 16
    batchsize = 32

    #dimensions = (batchsize, 13, 18, 1) # 2D
    #dimensions = (batchsize, 13, 18, 50, 1)  # 3D
    dimensions = (batchsize, 11, 13, 18, 50)  # 4D

    f = h5py.File(filepath, "r")
    filesize = len(f['y'])
    print(filesize)

    n_entries = 0
    while n_entries < (filesize - batchsize):
        xs = f['x'][n_entries : n_entries + batchsize]
        xs = np.reshape(xs, dimensions).astype(np.float32)

        y_values = f['y'][n_entries:n_entries+batchsize]
        y_values = np.reshape(y_values, (batchsize, y_values.shape[1]))
        ys = np.zeros((batchsize, 2), dtype=np.float32)

        for c, y_val in enumerate(y_values):
            ys[c] = y_val[0:2] # just for testing

        n_entries += batchsize
        #print n_entries
        yield (xs, ys)
    f.close()


number = 20
#t = timeit.timeit(generate_batches_from_hdf5_file, number = number)
#t = timeit.Timer(stmt="list(generate_batches_from_hdf5_file())", setup="from __main__ import generate_batches_from_hdf5_file")
#print t.timeit(number) / number
#print str(number) + 'loops, on average ' + str(t.timeit(number) / number *1000) + 'ms'

pr = cProfile.Profile()
pr.enable()

t = timeit.Timer(stmt="list(generate_batches_from_hdf5_file())", setup="from __main__ import generate_batches_from_hdf5_file")
print(str(number) + 'loops, on average ' + str(t.timeit(number) / number *1000) + 'ms')

pr.disable()

pr.print_stats(sort='time')

# TODO check with blosc


#!/usr/bin/env python
# coding=utf-8
# Filename: data_to_images.py
"""
Main OrcaSong code which takes raw simulated .h5 files and the corresponding .detx detector file as input in
order to generate 2D/3D/4D histograms ('images') that can be used for CNNs.

First argument: KM3NeT hdf5 simfile at JTE level.
Second argument: a .detx file that is associated with the hdf5 file.

The input file can be calibrated or not (e.g. contains pos_xyz of the hits) and the OrcaSong output is written
to the current folder by default (otherwise use --o option).
Makes only 4D histograms ('images') by default.

Usage:
    data_to_images.py [options] FILENAME DETXFILE
    data_to_images.py (-h | --help)

Options:
    -h --help                       Show this screen.

    -c CONFIGFILE                   Load all options from a config file (.toml format).

    --o OUTPUTPATH                  Path for the directory, where the OrcaSong output should be stored. [default: ./]

    --chunksize CHUNKSIZE           Chunksize (axis_0) that should be used for the hdf5 output of OrcaSong. [default: 32]

    --complib COMPLIB               Compression library that should be used for the OrcaSong output.
                                    All PyTables compression filters are available. [default: zlib]

    --complevel COMPLEVEL           Compression level that should be used for the OrcaSong output. [default: 1]

    --n_bins N_BINS                 Number of bins that are used in the image making for each dimension, e.g. (x,y,z,t).
                                    [default: 11,13,18,60]

    --det_geo DET_GEO               Which detector geometry to use for the binning, e.g. 'Orca_115l_23m_h_9m_v'.
                                    [default: Orca_115l_23m_h_9m_v]

    --do2d                          If 2D histograms, 'images', should be created.

    --do2d_plots                    If 2D pdf plot visualizations of the 2D histograms should be created, cannot be called if do2d=False.

    --do2d_plots_n N                For how many events the 2D plot visualizations should be made.
                                    OrcaSong will exit after reaching N events. [default: 10]

    --do3d                          If 3D histograms, 'images', should be created.

    --dont_do4d                     If 4D histograms, 'images', should NOT be created.

    --do4d_mode MODE                What dimension should be used in the 4D histograms as the 4th dim.
                                    Available: 'time', 'channel_id'. [default: time]

    --timecut_mode MODE             Defines what timecut mode should be used in hits_to_histograms.py.
                                    At the moment, these cuts are only optimized for ORCA 115l neutrino events!
                                    Currently available:
                                    'timeslice_relative': Cuts out the central 30% of the snapshot.
                                    'trigger_cluster': Cuts based on the mean of the triggered hits.
                                    The timespan for this cut can be chosen in --timecut_timespan.
                                    'None': No timecut.
                                    [default: trigger_cluster]

    --timecut_timespan TIMESPAN     Only used with timecut_mode 'trigger_cluster'.
                                    Defines the timespan of the trigger_cluster cut.
                                    Currently available:
                                    'all': [-350ns, 850ns] -> 20ns / bin (60 bins)
                                    'tight-1': [-250ns, 500ns] -> 12.5ns / bin
                                    'tight-2': [-150ns, 200ns] -> 5.8ns / bin
                                    [default: tight-1]

    --do_mc_hits                    If only the mc_hits (no BG hits!) should be used for the image processing.

    --data_cut_triggered            If non-triggered hits should be thrown away for the images.

    --data_cut_e_low E_LOW          Cut events that are lower than the specified threshold value in GeV.

    --data_cut_e_high E_HIGH        Cut events that are higher than the specified threshold value in GeV.

    --data_cut_throw_away FRACTION  Throw away a random fraction (percentage) of events. [default: 0.00]

    --prod_ident PROD_IDENT         Optional int identifier for the used mc production.
                                    This is useful, if you use events from two different mc productions,
                                    e.g. the 1-5GeV & 3-100GeV Orca 2016 MC. The prod_ident int will be saved in
                                    the 'y' dataset of the output file of OrcaSong. [default: 1]

"""

__author__ = 'Michael Moser'
__license__ = 'AGPL'
__version__ = '1.0'
__email__ = 'michael.m.moser@fau.de'
__status__ = 'Prototype'

import os
import sys
import warnings
import toml
from docopt import docopt
#from memory_profiler import profile # for memory profiling, call with @profile; myfunc()
#import line_profiler # call with kernprof -l -v file.py args
import km3pipe as kp
import km3modules as km
import matplotlib as mpl
mpl.use('Agg')
from matplotlib.backends.backend_pdf import PdfPages

from orcasong.file_to_hits import EventDataExtractor
from orcasong.hits_to_histograms import HistogramMaker


def parse_input():
    """
    Parses and returns all necessary input options for the data_to_images function.

    Check the data_to_images function to get docs about the individual parameters.
    """
    args = docopt(__doc__)

    if args['-c']:
        config = toml.load(args['-c'])
        args.update(config)

    fname = args['FILENAME']
    detx_filepath = args['DETXFILE']

    output_dirpath = args['--o']
    chunksize = int(args['--chunksize'])
    complib = args['--complib']
    complevel = int(args['--complevel'])
    n_bins = tuple(map(int, args['--n_bins'].split(',')))
    det_geo = args['--det_geo']
    do2d = args['--do2d']
    do2d_plots = (args['--do2d_plots'], int(args['--do2d_plots_n']))
    do3d = args['--do3d']
    do4d = (not bool(args['--dont_do4d']), args['--do4d_mode'])
    timecut = (args['--timecut_mode'], args['--timecut_timespan'])
    do_mc_hits = args['--do_mc_hits']
    data_cuts = dict()
    data_cuts['triggered'] = args['--data_cut_triggered']
    data_cuts['energy_lower_limit'] = float(args['--data_cut_e_low']) if args['--data_cut_e_low'] is not None else None
    data_cuts['energy_upper_limit'] = float(args['--data_cut_e_high']) if args['--data_cut_e_high'] is not None else None
    data_cuts['throw_away_prob'] = float(args['--data_cut_throw_away'])
    prod_ident = int(args['--prod_ident'])

    return fname, detx_filepath, output_dirpath, chunksize, complib, complevel, n_bins, det_geo, do2d,\
           do2d_plots, do3d, do4d, prod_ident, timecut, do_mc_hits, data_cuts


def parser_check_input(args):
    """
    Sanity check of the user input. Only necessary for options that are not boolean.

    Parameters
    ----------
    args : dict
        Docopt parser element that contains all input information.

    """
    #---- Checks input types ----#

    # Check for options with a single, non-boolean element
    single_args = {'--det_geo': str, '--do2d_plots_n': int, '--do4d_mode': str, '--timecut_mode': str,
                   'timecut_timespan': str,  '--data_cut_e_low ': float, '--data_cut_e_high': float,
                   '--data_cut_throw_away': float, '--prod_ident': int}

    for key in single_args:
        expected_arg_type = single_args[key]
        parsed_arg = args[key]

        if parsed_arg is None: # we don't want to check args when there has been no user input
            continue

        try:
            map(expected_arg_type, parsed_arg)
        except ValueError:
            raise TypeError('The argument option ', key, ' only accepts ', str(expected_arg_type),
                            ' values as an input.')

    # Checks the n_bins tuple input
    try:
        map(int, args['--n_bins'].split(','))
    except ValueError:
        raise TypeError('The argument option n_bins only accepts integer values as an input'
                        ' (Format: --n_bins 11,13,18,60).')


    # ---- Checks input types ----#

    # ---- Check other things ----#

    if not os.path.isfile(args['FILENAME']):
        raise IOError('The file -' + args['FILENAME'] + '- does not exist.')

    if not os.path.isfile(args['DETXFILE']):
        raise IOError('The file -' + args['DETXFILE'] + '- does not exist.')

    if bool(args['--do2d']) == False and bool(args['--do2d_plots']) == True:
        raise ValueError('The 2D pdf images cannot be created if do2d=False!')

    if bool(args['--do2d_plots']) == True and int(args['--do2d_plots_n']) > 100:
        warnings.warn('You declared do2d_pdf=(True, int) with int > 100. This will take more than two minutes.'
                      'Do you really want to create pdfs images for so many events?')


def make_output_dirs(output_dirpath, do2d, do3d, do4d):
    """
    Function that creates all output directories if they don't exist already.

    Parameters
    ----------
    output_dirpath : str
        Full path to the directory, where the orcasong output should be stored.
    do2d : bool
        Declares if 2D histograms, are to be created.
    do3d : bool
        Declares if 3D histograms are to be created.
    do4d : tuple(bool, str)
        Tuple that declares if 4D histograms should be created [0] and if yes, what should be used as the 4th dim after xyz.
        Currently, only 'time' and 'channel_id' are available.

    """
    if do2d:
        projections = ['xy', 'xz', 'yz', 'xt', 'yt', 'zt']
        for proj in projections:
            if not os.path.exists(output_dirpath + '/orcasong_output/4dTo2d/' + proj):
                os.makedirs(output_dirpath + '/orcasong_output/4dTo2d/' + proj)
    if do3d:
        projections = ['xyz', 'xyt', 'xzt', 'yzt', 'rzt']
        for proj in projections:
            if not os.path.exists(output_dirpath + '/orcasong_output/4dTo3d/' + proj):
                os.makedirs(output_dirpath + '/orcasong_output/4dTo3d/' + proj)
    if do4d[0]:
        proj = 'xyzt' if not do4d[1] == 'channel_id' else 'xyzc'
        if not os.path.exists(output_dirpath + '/orcasong_output/4dTo4d/' + proj):
            os.makedirs(output_dirpath + '/orcasong_output/4dTo4d/' + proj)


def calculate_bin_edges(n_bins, det_geo, detx_filepath, do4d):
    """
    Calculates the bin edges for the corresponding detector geometry (1 DOM/bin) based on the number of specified bins.

    Used later on for making the event "images" with the in the np.histogramdd funcs in hits_to_histograms.py.
    The bin edges are necessary in order to get the same bin size for each event regardless of the fact if all bins have a hit or not.

    Parameters
    ----------
    n_bins : tuple
        Contains the desired number of bins for each dimension, [n_bins_x, n_bins_y, n_bins_z].
    det_geo : str
        Declares what detector geometry should be used for the binning.
    detx_filepath : str
        Filepath of a .detx detector file which contains the geometry of the detector.
    do4d : tuple(boo, str)
        Tuple that declares if 4D histograms should be created [0] and if yes, what should be used as the 4th dim after xyz.

    Returns
    -------
    x_bin_edges, y_bin_edges, z_bin_edges : ndarray(ndim=1)
        The bin edges for the x,y,z direction.

    """
    # Loads a kp.Geometry object based on the filepath of a .detx file
    print("Reading detector geometry in order to calculate the detector dimensions from file " + detx_filepath)
    geo = kp.calib.Calibration(filename=detx_filepath)

    # derive maximum and minimum x,y,z coordinates of the geometry input [[xmin, ymin, zmin], [xmax, ymax, zmax]]
    dom_position_values = geo.get_detector().dom_positions.values()
    dom_pos_max = np.amax([pos for pos in dom_position_values], axis=0)
    dom_pos_min = np.amin([pos for pos in dom_position_values], axis=0)
    geo_limits = dom_pos_min, dom_pos_max
    print('Detector dimensions [[xmin, ymin, zmin], [xmax, ymax, zmax]]: ' + str(geo_limits))

    if det_geo == 'Orca_115l_23m_h_9m_v' or det_geo == 'Orca_115l_23m_h_?m_v':
        x_bin_edges = np.linspace(geo_limits[0][0] - 9.95, geo_limits[1][0] + 9.95, num=n_bins[0] + 1) #try to get the lines in the bin center 9.95*2 = average x-separation of two lines
        y_bin_edges = np.linspace(geo_limits[0][1] - 9.75, geo_limits[1][1] + 9.75, num=n_bins[1] + 1) # Delta y = 19.483

        # Fitted offsets: x,y,factor: factor*(x+x_off), # Stefan's modifications:
        offset_x, offset_y, scale = [6.19, 0.064, 1.0128]
        x_bin_edges = (x_bin_edges + offset_x) * scale
        y_bin_edges = (y_bin_edges + offset_y) * scale

        if det_geo == 'Orca_115l_23m_h_?m_v':
            # ORCA denser detector study
            z_bin_edges = np.linspace(37.84 - 7.5, 292.84 + 7.5, num=n_bins[2] + 1)  # 15m vertical, 18 DOMs
            # z_bin_edges = np.linspace(37.84 - 6, 241.84 + 6, num=n_bins[2] + 1)  # 12m vertical, 18 DOMs
            # z_bin_edges = np.linspace(37.84 - 4.5, 190.84 + 4.5, num=n_bins[2] + 1)  # 9m vertical, 18 DOMs
            # z_bin_edges = np.linspace(37.84 - 3, 139.84 + 3, num=n_bins[2] + 1)  # 6m vertical, 18 DOMs
            # z_bin_edges = np.linspace(37.84 - 2.25, 114.34 + 2.25, num=n_bins[2] + 1)  # 4.5m vertical, 18 DOMs

        else:
            n_bins_z = n_bins[2] if do4d[1] != 'xzt-c' else n_bins[1] # n_bins = [xyz,t/c] or n_bins = [xzt,c]
            z_bin_edges = np.linspace(geo_limits[0][2] - 4.665, geo_limits[1][2] + 4.665, num=n_bins_z + 1)  # Delta z = 9.329

        # calculate_bin_edges_test(dom_positions, y_bin_edges, z_bin_edges) # test disabled by default. Activate it, if you change the offsets in x/y/z-bin-edges

    else:
        raise ValueError('The specified detector geometry "' + str(det_geo) + '" is not available.')

    return geo, x_bin_edges, y_bin_edges, z_bin_edges


def calculate_bin_edges_test(geo, y_bin_edges, z_bin_edges):
    """
    Tests, if the bins in one direction don't accidentally have more than 'one' OM.

    For the x-direction, an overlapping can not be avoided in an orthogonal space.
    For y and z though, it can!
    For y, every bin should contain the number of lines per y-direction * 18 for 18 doms per line.
    For z, every bin should contain 115 entries, since every z bin contains one storey of the 115 ORCA lines.
    Not 100% accurate, since only the dom positions are used and not the individual pmt positions for a dom.
    """
    dom_positions = np.stack(list(geo.get_detector().dom_positions.values()))
    dom_y = dom_positions[:, 1]
    dom_z = dom_positions[:, 2]
    hist_y = np.histogram(dom_y, bins=y_bin_edges)
    hist_z = np.histogram(dom_z, bins=z_bin_edges)

    print('----------------------------------------------------------------------------------------------')
    print('Y-axis: Bin content: ' + str(hist_y[0]))
    print('It should be:        ' + str(np.array(
        [4 * 18, 7 * 18, 9 * 18, 10 * 18, 9 * 18, 10 * 18, 10 * 18, 10 * 18, 11 * 18, 10 * 18, 9 * 18, 8 * 18, 8 * 18])))
    print('Y-axis: Bin edges: ' + str(hist_y[1]))
    print('..............................................................................................')
    print('Z-axis: Bin content: ' + str(hist_z[0]))
    print('It should have 115 entries everywhere')
    print('Z-axis: Bin edges: ' + str(hist_z[1]))
    print('----------------------------------------------------------------------------------------------')


def get_file_particle_type(fname):
    """
    Returns a string that specifies the type of the particles that are contained in the input file.

    Parameters
    ----------
    fname : str
        Filename (full path!) of the input file.

    Returns
    -------
    file_particle_type : str
        String that specifies the type of particles that are contained in the file: ['undefined', 'muon', 'neutrino'].

    """
    event_pump = kp.io.hdf5.HDF5Pump(filename=fname) # TODO suppress print of hdf5pump and close pump afterwards

    if 'McTracks' not in event_pump[0]:
        file_particle_type = 'undefined'
    else:
        particle_type = event_pump[0]['McTracks'].type

        # if mupage file: first mc_track is an empty neutrino track, second track is the first muon
        if particle_type[0] == 0 and np.abs(particle_type[1]) == 13:
            file_particle_type = 'muon'

        # the first mc_track is the primary neutrino if the input file is containing neutrino events
        elif np.abs(particle_type[0]) in [12, 14, 16]:
            file_particle_type = 'neutrino'
        else:
            raise ValueError('The type of the particles in the "McTracks" folder, <', str(particle_type), '> is not known.')

    event_pump.close_file()

    return file_particle_type


class EventSkipper(kp.Module):
    """
    KM3Pipe module that skips events based on some data_cuts.
    """
    def configure(self):
        self.data_cuts = self.require('data_cuts')

    def process(self, blob):
        continue_bool = skip_event(blob['event_track'], self.data_cuts)
        if continue_bool:
            return
        else:
            return blob


def skip_event(event_track, data_cuts):
    """
    Function that checks if an event should be skipped, depending on the data_cuts input.

    Parameters
    ----------
    event_track : ndarray(ndim=1)
        1D array containing important MC information of the event, only the energy of the event (pos_2) is used here.
    data_cuts : dict
        Dictionary that contains information about any possible cuts that should be applied.
        Supports the following cuts: 'triggered', 'energy_lower_limit', 'energy_upper_limit', 'throw_away_prob'.

    Returns
    -------
    continue_bool : bool
        boolean flag to specify, if this event should be skipped or not.

    """
    continue_bool = False

    if data_cuts['energy_lower_limit'] is not None:
        continue_bool = event_track.energy[0] < data_cuts['energy_lower_limit'] # True if E < lower limit

    if data_cuts['energy_upper_limit'] is not None and continue_bool == False:
        continue_bool = event_track.energy[0] > data_cuts['energy_upper_limit'] # True if E > upper limit

    if data_cuts['throw_away_prob'] > 0 and continue_bool == False:
        throw_away_prob = data_cuts['throw_away_prob']
        throw_away = np.random.choice([False, True], p=[1 - throw_away_prob, throw_away_prob])
        if throw_away: continue_bool = True

    return continue_bool


def data_to_images(fname, detx_filepath, output_dirpath, chunksize, complib, complevel, n_bins, det_geo, do2d, do2d_plots, do3d, do4d, prod_ident, timecut,
                   do_mc_hits, data_cuts):
    """
    Main code with config parameters. Reads raw .hdf5 files and creates 2D/3D histogram projections that can be used
    for a CNN.

    Parameters
    ----------
    fname : str
        Filename (full path!) of the input file.
    detx_filepath : str
        String with the full filepath to the corresponding .detx file of the input file.
        Used for the binning and for the hits calibration if the input file is not calibrated yet
        (e.g. hits do not contain pos_x/y/z, time, ...).
    output_dirpath : str
        Full path to the directory, where the orcasong output should be stored.
    chunksize : int
        Chunksize (along axis_0) that is used for saving the OrcaSong output to a .h5 file.
    complib : str
        Compression library that is used for saving the OrcaSong output to a .h5 file.
        All PyTables compression filters are available, e.g. 'zlib', 'lzf', 'blosc', ... .
    complevel : int
        Compression level for the compression filter that is used for saving the OrcaSong output to a .h5 file.
    n_bins : tuple of int
        Declares the number of bins that should be used for each dimension, e.g. (x,y,z,t).
    det_geo : str
        Declares what detector geometry should be used for the binning. E.g. 'Orca_115l_23m_h_9m_v'.
    do2d : bool
        Declares if 2D histograms, 'images', should be created.
    do2d_plots : tuple(bool, int)
        Declares if pdf visualizations of the 2D histograms should be created, cannot be called if do2d=False.
        The event loop will be stopped after the integer specified in the second argument.
    do3d : bool
        Declares if 3D histograms should be created.
    do4d : tuple(bool, str)
        Tuple that declares if 4D histograms should be created [0] and if yes, what should be used as the 4th dim after xyz.
        Currently, only 'time' and 'channel_id' are available.
    prod_ident : int
        Optional int identifier for the used mc production.
        This is e.g. useful, if you use events from two different mc productions, e.g. the 1-5GeV & 3-100GeV Orca 2016 MC.
        In this case, the events are not fully distinguishable with only the run_id and the event_id!
        In order to keep a separation, an integer can be set in the event_track for all events, such that they stay distinguishable.
    timecut : tuple(str, str/None)
        Tuple that defines what timecut should be used in hits_to_histograms.py.
        Currently available:
        ('timeslice_relative', None): Cuts out the central 30% of the snapshot.
        ('trigger_cluster', 'all' / 'tight-1' / 'tight-2'): Cuts based on the mean of the triggered hits.
        (None, ...): No timecut.
        all: [-350ns, 850ns] -> 20ns / bin (60 bins)
        tight-1: [-250ns, 500ns] -> 12.5ns / bin , tight-2: [-150ns, 200ns] -> 5.8ns / bin
    do_mc_hits : bool
        Declares if hits (False, mc_hits + BG) or mc_hits (True) should be processed.
    data_cuts : dict
        Dictionary that contains information about any possible cuts that should be applied.
        Supports the following cuts: 'triggered', 'energy_lower_limit', 'energy_upper_limit', 'throw_away_prob'.

    """
    make_output_dirs(output_dirpath, do2d, do3d, do4d)

    filename = os.path.basename(os.path.splitext(fname)[0])
    filename_output = filename.replace('.','_')
    km.GlobalRandomState(seed=42)  # set random km3pipe (=numpy) seed

    geo, x_bin_edges, y_bin_edges, z_bin_edges = calculate_bin_edges(n_bins, det_geo, detx_filepath, do4d)
    pdf_2d_plots = PdfPages(output_dirpath + '/orcasong_output/4dTo2d/' + filename_output + '_plots.pdf') if do2d_plots[0] is True else None

    file_particle_type = get_file_particle_type(fname)

    print('Generating histograms from the hits for files based on ' + fname)

    # Initialize OrcaSong Event Pipeline

    pipe = kp.Pipeline()
    pipe.attach(km.common.StatusBar, every=50)
    pipe.attach(kp.io.hdf5.HDF5Pump, filename=fname)
    pipe.attach(km.common.Keep, keys=['EventInfo', 'Header', 'RawHeader', 'McTracks', 'Hits', 'McHits'])
    pipe.attach(EventDataExtractor,
                file_particle_type=file_particle_type, geo=geo, do_mc_hits=do_mc_hits,
                data_cuts=data_cuts, do4d=do4d, prod_ident=prod_ident)
    pipe.attach(km.common.Keep, keys=['event_hits', 'event_track'])
    pipe.attach(EventSkipper, data_cuts=data_cuts)
    pipe.attach(HistogramMaker,
                x_bin_edges=x_bin_edges, y_bin_edges=y_bin_edges, z_bin_edges=z_bin_edges,
                n_bins=n_bins, timecut=timecut, do2d=do2d, do2d_plots=do2d_plots, pdf_2d_plots=pdf_2d_plots,
                do3d=do3d, do4d=do4d)
    pipe.attach(km.common.Delete, keys=['event_hits'])

    if do2d:
        for proj in ['xy', 'xz', 'yz', 'xt', 'yt', 'zt']:
            savestr = output_dirpath + '/orcasong_output/4dTo2d/' + proj + '/' + filename_output + '_' + proj + '.h5'
            pipe.attach(kp.io.HDF5Sink, filename=savestr, blob_keys=[proj, 'event_track'], complib=complib, complevel=complevel, chunksize=chunksize)


    if do3d:
        for proj in ['xyz', 'xyt', 'xzt', 'yzt', 'rzt']:
            savestr = output_dirpath + '/orcasong_output/4dTo3d/' + proj + '/' + filename_output + '_' + proj + '.h5'
            pipe.attach(kp.io.HDF5Sink, filename=savestr, blob_keys=[proj, 'event_track'], complib=complib, complevel=complevel, chunksize=chunksize)

    if do4d[0]:
        proj = 'xyzt' if not do4d[1] == 'channel_id' else 'xyzc'
        savestr = output_dirpath + '/orcasong_output/4dTo4d/' + proj + '/' + filename_output + '_' + proj + '.h5'
        pipe.attach(kp.io.HDF5Sink, filename=savestr, blob_keys=[proj, 'event_track'], complib=complib, complevel=complevel, chunksize=chunksize)

    # Execute Pipeline
    pipe.drain()


def main():
    """
    Parses the input to the main data_to_images function.
    """
    fname, detx_filepath, output_dirpath, chunksize, complib, complevel, n_bins, det_geo, do2d, do2d_plots, do3d,\
    do4d, prod_ident, timecut, do_mc_hits, data_cuts = parse_input()
    data_to_images(fname, detx_filepath, output_dirpath, chunksize, complib, complevel, n_bins, det_geo, do2d,
                   do2d_plots, do3d, do4d, prod_ident, timecut, do_mc_hits, data_cuts)


if __name__ == '__main__':
    main()













# -*- coding: utf-8 -*-

import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
mpl.rcParams.update({'font.size': 22})
z_layer_no = 5  # number of z layer to plot xy from, bottom to top
geo_file = "/home/woody/capn/mppi033h/Code/HPC/h5ToHisto/ORCA_Geo_115lines.txt"
n_bins = (11, 13, 18)

# [id,x,y,z]
geo = np.loadtxt(geo_file)

x = geo[:, 1]
y = geo[:, 2]
z = geo[:, 3]

which_z_layer = z == np.unique(z)[z_layer_no]
geo_reduced = geo[which_z_layer, :]
x_red = geo_reduced[:, 1]
y_red = geo_reduced[:, 2]


def calculate_bin_edges(n_bins, geo):
    """
    Calculates the bin edges for the later np.histogramdd actions based on the number of specified bins.
    This is performed in order to get the same bin size for each event regardless of the fact if all bins have a hit or not.
    :param tuple n_bins: contains the desired number of bins for each dimension. [n_bins_x, n_bins_y, n_bins_z]
    :param str fname_geo_limits: filepath of the .txt ORCA geometry file.
    :return: ndarray(ndim=1) x_bin_edges, y_bin_edges, z_bin_edges: contains the resulting bin edges for each dimension.
    """
    # Gefittete offsets: x,y,factor: factor*(x+x_off)
    # [6.19, 0.064, 1.0128]

    # print "Reading detector geometry in order to calculate the detector dimensions from file " + fname_geo_limits
    # geo = np.loadtxt(fname_geo_limits)

    # derive maximum and minimum x,y,z coordinates of the geometry input [[first_OM_id, xmin, ymin, zmin], [last_OM_id, xmax, ymax, zmax]]
    geo_limits = np.nanmin(geo, axis=0), np.nanmax(geo, axis=0)
    # print ('Detector dimensions [[first_OM_id, xmin, ymin, zmin], [last_OM_id, xmax, ymax, zmax]]: ' + str(geo_limits))

    x_bin_edges = np.linspace(geo_limits[0][1] - 9.95, geo_limits[1][1] + 9.95, num=n_bins[
                                                                                        0] + 1)  # try to get the lines in the bin center 9.95*2 = average x-separation of two lines
    y_bin_edges = np.linspace(geo_limits[0][2] - 9.75, geo_limits[1][2] + 9.75, num=n_bins[1] + 1)  # Delta y = 19.483
    z_bin_edges = np.linspace(geo_limits[0][3] - 4.665, geo_limits[1][3] + 4.665, num=n_bins[2] + 1)  # Delta z = 9.329

    # offset_x, offset_y, scale = [6.19, 0.064, 1.0128]
    # x_bin_edges = (x_bin_edges + offset_x )*scale
    # y_bin_edges = (y_bin_edges + offset_y )*scale

    # calculate_bin_edges_test(geo, y_bin_edges, z_bin_edges) # test disabled by default. Activate it, if you change the offsets in x/y/z-bin-edges

    return x_bin_edges, y_bin_edges, z_bin_edges


x_bin_edges, y_bin_edges, z_bin_edges = calculate_bin_edges(n_bins, geo)


def calculate_bin_stats(x_one_layer, y_one_layer, x_bin_edges, y_bin_edges):
    hist_xy = np.histogram2d(x_one_layer, y_one_layer, bins=(x_bin_edges, y_bin_edges))[0]
    unique, counts = np.unique(hist_xy.flatten(), return_counts=True)
    return unique, counts


def scan_for_optimum(x_one_layer, y_one_layer, n_bins, geo):
    x_edges, y_edges, z_edges = calculate_bin_edges(n_bins, geo)
    current_unique, current_counts = calculate_bin_stats(x_one_layer, y_one_layer, x_edges, y_edges)

    result_matrix = np.zeros((3, 10, 10), dtype=int)

    for x_off in np.arange(-5, 5, 1):
        for y_off in np.arange(-5, 5, 1):
            unique, counts = calculate_bin_stats(x_one_layer, y_one_layer, x_edges + x_off, y_edges + y_off)
            result_matrix[0, x_off + 5, y_off + 5] = counts[0]
            result_matrix[1, x_off + 5, y_off + 5] = counts[1]
            result_matrix[2, x_off + 5, y_off + 5] = counts[2] if len(counts) >= 3 else 0
            print(x_off, y_off, "\t", counts)
    #for i in range(len(result_matrix[0])):
        #for j in range(len(result_matrix[0, 0])):
            #print (result_matrix[:, i, j], end="")
        #print("")


def get_distance_to_edges(x_edges, y_edges, x_one_layer, y_one_layer):
    # Return the distance of all doms

    # The size of the bins:
    box_length_x = x_edges[1] - x_edges[0]
    box_length_y = y_edges[1] - y_edges[0]

    # The position of the doms in the unit box from (0,0) to (box_length_x,box_length_y)
    x_unit_cell = (x_one_layer - np.min(x_edges)) % box_length_x
    y_unit_cell = (y_one_layer - np.min(y_edges)) % box_length_y

    center = [box_length_x / 2, box_length_y / 2]
    # euclidean distance to center
    distances = np.sqrt((x_unit_cell - center[0]) ** 2 + (y_unit_cell - center[1]) ** 2)
    # l1 distance to center
    # distances = np.abs(x_unit_cell - center[0]) + np.abs(y_unit_cell - center[1])

    # Minimum distance to edge
    # distances = -1 * np.min([np.min(x_unit_cell), np.min(box_length_x-x_unit_cell), np.min(y_unit_cell), np.min(box_length_y-y_unit_cell)] )

    return np.mean(distances)


def scan_for_highest_distance(x_one_layer, y_one_layer, x_bin_edges, y_bin_edges, x_offsets, y_offsets, multiply=[1],
                              efficient=False):
    # Shift the grid around by x/y offsets in meters, and get the distance to the bin center
    # Also returns the highest number of doms per bin:
    # return [[x_off],[y_off],[distance],[max_doms_per_bin]]

    i = 0
    total_offsets_to_check = len(x_offsets) * len(y_offsets) * len(multiply)
    if efficient == False:
        results = np.zeros((5, total_offsets_to_check))  # [[x_off],[y_off],[distance],[max_doms_per_bin], [factor]
    else:
        results = np.array([0.0, 0.0, 1000.0, 0.0, 0.0])

    for x_offset in x_offsets:
        for y_offset in y_offsets:
            for factor in multiply:
                x_edges = factor * (x_bin_edges + x_offset)
                y_edges = factor * (y_bin_edges + y_offset)

                hist_xy = np.histogram2d(x_one_layer, y_one_layer, bins=(x_edges, y_edges))[0]
                max_bins_per_dom = hist_xy.flatten().max()
                distance = get_distance_to_edges(x_edges, y_edges, x_one_layer, y_one_layer)

                if efficient == False:
                    results[:, i] = (x_offset, y_offset, distance, max_bins_per_dom, factor)
                else:
                    if distance < results[2] and max_bins_per_dom == 1:
                        results = (x_offset, y_offset, distance, max_bins_per_dom, factor)

                # if i % 5000 == 0:
                #     print(i, "/", total_offsets_to_check)
                i += 1
    return results


def plot_3d(x, y, z):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(x, y, z, c='r', marker='o')
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    plt.show()


def plot_2d(x_bin_edges, y_bin_edges, x_one_layer, y_one_layer):
    # xy_one_layer: x and y coordinates of doms of one z layer
    box_length_x = x_bin_edges[1] - x_bin_edges[0]
    box_length_y = y_bin_edges[1] - y_bin_edges[0]
    hist_xy = np.histogram2d(x_one_layer, y_one_layer, bins=(x_bin_edges, y_bin_edges))[0]
    #print("Doms per bin:", calculate_bin_stats(x_one_layer, y_one_layer, x_bin_edges, y_bin_edges))

    # max_doms_inside=hist_xy.max() #2
    # min_doms_inside=hist_xy.min() #0

    fig = plt.figure(figsize=(8, 13))
    ax = fig.add_subplot(111)
    #ax = plt.subplot(111, adjustable='box-forced')


    for x_bin_edge in x_bin_edges:
        ax.plot([x_bin_edge, x_bin_edge], [y_bin_edges.min(), y_bin_edges.max()], color="black", ls="-", zorder=-1)
    for y_bin_edge in y_bin_edges:
        ax.plot([x_bin_edges.min(), x_bin_edges.max()], [y_bin_edge, y_bin_edge], color="black", ls="-", zorder=-1)

    for bin_no_x, x_bin_edge in enumerate(x_bin_edges[:-1]):
        for bin_no_y, y_bin_edge in enumerate(y_bin_edges[:-1]):
            alpha_max = 0.3
            doms_inside = hist_xy[bin_no_x, bin_no_y]
            alpha = doms_inside * alpha_max / 2
            # alpha = (doms_inside-min_doms_inside) * alpha_max/max_doms_inside
            ax.add_patch(
                Rectangle([x_bin_edge, y_bin_edge], box_length_x, box_length_y, fc="blue", alpha=alpha, zorder=-2))

    plt.rcParams.update({'font.size': 16})
    ax.scatter(x_one_layer, y_one_layer, c='r', marker='o', label="DOM lines", zorder=1)
    ax.set_xlabel('X (m)')
    ax.minorticks_on()
    ax.set_ylabel('Y (m)')
    ax.set_aspect("equal")

    new_tick_locations_x = x_bin_edges[:-1] + 0.5 * box_length_x
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())
    ax2.set_xticks(new_tick_locations_x)
    ax2.set_xticklabels(np.arange(1, n_bins[0] + 1, 1))
    ax2.set_xlabel("x bin no.")
    ax2.set_aspect("equal")

    new_tick_locations_y = y_bin_edges[:-1] + 0.5 * box_length_y
    ax3 = ax.twinx()
    ax3.set_ylim(ax.get_ylim())
    ax3.set_yticks(new_tick_locations_y)
    ax3.set_yticklabels(np.arange(1, n_bins[1] + 1, 1))
    ax3.set_ylabel("y bin no.")
    ax3.set_aspect("equal")

    legend = ax.legend(loc="lower right")
    legend.get_frame().set_alpha(1)
    fig.suptitle("Dom locations and Binning in XY direction")
    #ax.set_ylim(-200, 200)
    #plt.setp(ax, aspect=1.0, adjustable='box-forced')
    #plt.show()
    plt.savefig('binning.pdf')


# scan_for_optimum(x_red,y_red, n_bins, geo)

def maximize_distance():
    x_offsets = np.linspace(-10, 10, 200 + 1)
    y_offsets = np.linspace(-2, 6, 2000 + 1)
    # multiply=np.linspace(1.01,1.02, 50+1)
    multiply = [1]
    x_offsets = [6.19]

    efficient = True

    distances = scan_for_highest_distance(x_red, y_red, x_bin_edges, y_bin_edges, x_offsets, y_offsets, multiply,
                                          efficient)

    # [[x_off],[y_off],[distance],[max_doms_per_bin]] ... (4, ... )

    def surface_plot():
        fig = plt.figure()
        ax = fig.gca(projection='3d')

        """
        for i in range(1,int(distances[3].max())+1):
            i_max_doms_per_bin = distances[:,distances[3]==i]
            surf=ax.scatter(i_max_doms_per_bin[0], i_max_doms_per_bin[1], i_max_doms_per_bin[2], label=str(i))
        """

        ax.plot_trisurf(distances[0], distances[1], distances[2])

        ax.set_xlabel('X offset (m)')
        ax.set_ylabel('Y offset (m)')
        ax.set_zlabel('Extremal distance to edges')
        plt.legend()
        plt.show()

    # surface_plot()
    if efficient == False:
        one_dom_per_bin = distances[:, distances[3] == 1]
        best_x, best_y, best_dist, doms_per_bin, best_factor = one_dom_per_bin[:, np.argmin(one_dom_per_bin[2])]
    else:
        best_x, best_y, best_dist, doms_per_bin, best_factor = distances

    # print("Best x-offset:", best_x, "Best y-offset", best_y, "Best factor: ", best_factor, "Distance at this point:",
    #       best_dist)
    return (best_x, best_y, best_factor)


def show_distance_of_bins(x_one_layer, y_one_layer, x_bin_edges, y_bin_edges, offsets):
    for offset in offsets:
        x_edges = offset[2] * (x_bin_edges + offset[0])
        y_edges = offset[2] * (y_bin_edges + offset[1])

        box_length_x = x_edges[1] - x_edges[0]
        box_length_y = y_edges[1] - y_edges[0]

        x_unit_cell = (x_one_layer - np.min(x_edges)) % box_length_x
        y_unit_cell = (y_one_layer - np.min(y_edges)) % box_length_y

        distances = np.sort(
            np.min([x_unit_cell, box_length_x - x_unit_cell, y_unit_cell, box_length_y - y_unit_cell], axis=0))
        plt.plot(distances, label="x: " + str(offset[0]) + " y: " + str(offset[1]) + " k: " + str(offset[2]))

    plt.xlabel("Dom number")
    plt.ylabel("Minimal distance to bin edge (m)")
    plt.xlim([0, 114])
    plt.ylim([0, box_length_x / 2])
    plt.grid()
    plt.legend()
    plt.show()


def get_distance_to_edges_minimizer(x, args):
    # Return the distance of all doms
    x_offset, y_offset, factor = x
    x_bin_edges, y_bin_edges, x_one_layer, y_one_layer = args

    x_edges = factor * (x_bin_edges + x_offset)
    y_edges = factor * (y_bin_edges + y_offset)

    # The size of the bins:
    box_length_x = x_edges[1] - x_edges[0]
    box_length_y = y_edges[1] - y_edges[0]

    # The position of the doms in the unit box from (0,0) to (box_length_x,box_length_y)
    x_unit_cell = (x_one_layer - np.min(x_edges)) % box_length_x
    y_unit_cell = (y_one_layer - np.min(y_edges)) % box_length_y

    distance = -1 * np.min([np.min(x_unit_cell), np.min(box_length_x - x_unit_cell), np.min(y_unit_cell),
                            np.min(box_length_y - y_unit_cell)])

    return distance


# res = minimize( get_distance_to_edges_minimizer, x0=[0,0,1], args=[x_bin_edges, y_bin_edges, x_red, y_red], method="Powell")


# best_x, best_y, best_factor = maximize_distance()


# gute Offsets:
# 6.45,-4.25     Distance: -2.7355
# x,y,factor
offset_array = [[0, 0, 1], [6.45, -4.25, 1], [6.19, 0.064, 1.0128], [-2.2, -0.1, 1]]

# show_distance_of_bins(x_red, y_red, x_bin_edges, y_bin_edges, offset_array )

plot_offset = offset_array[2]
# plot_offset=[best_x, best_y, best_factor]
plot_2d(plot_offset[2] * (x_bin_edges + plot_offset[0]), plot_offset[2] * (y_bin_edges + plot_offset[1]), x_red, y_red)



#!/usr/bin/env python
from setuptools import setup, find_packages
from pkg_resources import get_distribution, DistributionNotFound

with open('requirements.txt') as fobj:
    requirements = [l.strip() for l in fobj.readlines()]

setup(
    name='orcasong',
    description='Makes images for a NN based on the hit information of neutrino events in the neutrino telescope KM3NeT',
    url='https://git.km3net.de/ml/OrcaSong',
    author='Michael Moser',
    author_email='mmoser@km3net.de, michael.m.moser@fau.de',
    license='AGPL',
    install_requires=requirements,
    packages=find_packages(),
    include_package_data=True,
    entry_points={'console_scripts': ['make_nn_images=orcasong.data_to_images:main']},
    setup_requires=['setuptools_scm'],
    use_scm_version={
        'write_to': 'orcasong/version.txt',
        'tag_regex': r'^(?P<prefix>v)?(?P<version>[^\+]+)(?P<suffix>.*)?$',
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python',
    ]

)

__author__ = 'Michael Moser'


import h5py
import numpy as np

path = '/home/woody/capn/mppi033h/Data/ORCA_JTE_NEMOWATER/ip_images_1-100GeV/4dTo4d/time_-250+500_w_gf_60b'
# JTE_KM3Sim_gseagen_muon-CC_1-5GeV-9_2E5-1bin-1_0gspec_ORCA115_9m_2016_98_xyzt.h5
ptypes = {'muon-CC': 'JTE_KM3Sim_gseagen_muon-CC_1-5GeV-9_2E5-1bin-1_0gspec_ORCA115_9m_2016_',
          'elec-CC': 'JTE_KM3Sim_gseagen_elec-CC_1-5GeV-2_7E5-1bin-1_0gspec_ORCA115_9m_2016_'}

event_id, run_id = None, None
for ptype in ptypes.keys():
    for i in range(601):
        if i % 100 == 0:
            print(i)
        if i == 0: continue

        f = h5py.File(path + '/' + ptypes[ptype] + str(i) + '_xyzt.h5', 'r')
        event_id_f = f['y'][:, 0]
        run_id_f = f['y'][:, 9]

        if event_id is None:
            event_id = event_id_f
            run_id = run_id_f
        else:
            event_id = np.concatenate([event_id, event_id_f], axis=0)
            run_id = np.concatenate([run_id, run_id_f], axis=0)

        f.close()

    ax = np.newaxis
    arr = np.concatenate([run_id[:, ax], event_id[:, ax]], axis=1)
    np.save('/home/woody/capn/mppi033h/Code/OrcaSong/utilities/low_e_prod_surviving_evts_' + ptype + '.npy', arr)
    event_id, run_id = None, None




